@@ -65,6 +65,7 @@ bool isCloneable(WidgetBase::WidgetType type)
     case WidgetBase::wt_Node:
     case WidgetBase::wt_Artifact:
     case WidgetBase::wt_Instance:
+    case WidgetBase::wt_Entity:
         return true;
     default:
         return false;
@@ -84,6 +85,7 @@ UMLObject* findObjectInList(Uml::ID::Type id, const UMLObjectList& inList)
 {
     for (UMLObjectListIt oit(inList); oit.hasNext();) {
         UMLObject *obj = oit.next();
+        uIgnoreZeroPointer(obj);
         if (obj->id() == id)
             return obj;
         UMLObject *o;
@@ -326,6 +328,35 @@ UMLObject* findUMLObjectRaw(const UMLObjectList& inList,
     return 0;
 }
 
+/**
+ * Find the UML object of the given type and name in the passed-in list.
+ * This method searches for the raw name.
+ *
+ * @param inList        List in which to seek the object.
+ * @param name          Name of the object to find.
+ * @param type          ObjectType of the object to find (optional.)
+ *                      When the given type is ot_UMLObject the type is
+ *                      disregarded, i.e. the given name is the only
+ *                      search criterion.
+ * @return      Pointer to the UMLObject found, or NULL if not found.
+ */
+UMLObject* findUMLObjectRecursive(const UMLObjectList& inList,
+                                  const QString& name,
+                                  UMLObject::ObjectType type /* = ot_UMLObject */)
+{
+    foreach(UMLObject *obj, inList) {
+        if (obj->name() == name && type == obj->baseType())
+            return obj;
+        UMLPackage *pkg = obj->asUMLPackage();
+        if (pkg && pkg->containedObjects().size() > 0) {
+            UMLObject *o = findUMLObjectRecursive(pkg->containedObjects(), name, type);
+            if (o)
+                return o;
+        }
+    }
+    return 0;
+}
+
 /**
  * Get the root folder of the given UMLObject.
  */
@@ -435,7 +466,7 @@ UMLPackage* treeViewGetPackageFromCurrent()
 /**
  * Build the diagram name from the tree view.
 
- * The function returns a relative path constructed from the folder hierachy.
+ * The function returns a relative path constructed from the folder hierarchy.
  * @param id   the id of the diaram
  * @return     the constructed diagram name
  */
@@ -1142,24 +1173,29 @@ bool typeIsAllowedInType(UMLListViewItem::ListViewType childType,
                parentType == UMLListViewItem::lvt_UseCase_View;
     case UMLListViewItem::lvt_Subsystem:
         return parentType == UMLListViewItem::lvt_Component_Folder ||
-               parentType == UMLListViewItem::lvt_Subsystem;
+               parentType == UMLListViewItem::lvt_Subsystem ||
+               parentType == UMLListViewItem::lvt_Component_View;
     case UMLListViewItem::lvt_Component:
-    case UMLListViewItem::lvt_Port:
         return parentType == UMLListViewItem::lvt_Component_Folder ||
                parentType == UMLListViewItem::lvt_Component ||
+               parentType == UMLListViewItem::lvt_Subsystem ||
+               parentType == UMLListViewItem::lvt_Component_View;
+    case UMLListViewItem::lvt_Port:
+        return parentType == UMLListViewItem::lvt_Component ||
                parentType == UMLListViewItem::lvt_Subsystem;
     case UMLListViewItem::lvt_Artifact:
     case UMLListViewItem::lvt_Component_Diagram:
         return parentType == UMLListViewItem::lvt_Component_Folder ||
                parentType == UMLListViewItem::lvt_Component_View;
-        break;
     case UMLListViewItem::lvt_Node:
     case UMLListViewItem::lvt_Deployment_Diagram:
-        return parentType == UMLListViewItem::lvt_Deployment_Folder;
+        return parentType == UMLListViewItem::lvt_Deployment_Folder ||
+               parentType == UMLListViewItem::lvt_Deployment_View;
     case UMLListViewItem::lvt_Entity:
     case UMLListViewItem::lvt_EntityRelationship_Diagram:
     case UMLListViewItem::lvt_Category:
-        return parentType == UMLListViewItem::lvt_EntityRelationship_Folder;
+        return parentType == UMLListViewItem::lvt_EntityRelationship_Folder ||
+               parentType == UMLListViewItem::lvt_EntityRelationship_Model;
     default:
         return false;
     }
@@ -1838,7 +1874,7 @@ Uml::DiagramType::Enum convert_LVT_DT(UMLListViewItem::ListViewType lvt)
 /**
  * Converts a list view type enum to the equivalent settings dialog type.
  *
- * @param lvt   The ListViewType to convert.
+ * @param type   The ListViewType to convert.
  * @return  The converted settings dialog type
  */
 MultiPageDialogBase::PageType convert_LVT_PT(UMLListViewItem::ListViewType type)
@@ -2008,7 +2044,9 @@ bool typeIsAllowedInDiagram(UMLObject* o, UMLScene *scene)
         }
         break;
     case Uml::DiagramType::EntityRelationship:
-        if (ot != UMLObject::ot_Entity && ot != UMLObject::ot_Category)
+        if (scene->widgetOnDiagram(id) ||
+            (ot != UMLObject::ot_Entity &&
+             ot != UMLObject::ot_Category))
             bAccept = false;
         break;
     default:
@@ -2063,5 +2101,30 @@ bool typeIsAllowedInDiagram(UMLWidget* w, UMLScene *scene)
     return bAccept;
 }
 
+/**
+ * return true if given object type supports associatons
+ * @param type uml object type to check
+ */
+bool hasAssociations(UMLObject::ObjectType type)
+{
+    switch (type) {
+        case UMLObject::ot_Actor:
+        case UMLObject::ot_UseCase:
+        case UMLObject::ot_Class:
+        case UMLObject::ot_Package:
+        case UMLObject::ot_Component:
+        case UMLObject::ot_Node:
+        case UMLObject::ot_Artifact:
+        case UMLObject::ot_Interface:
+        case UMLObject::ot_Enum:
+        case UMLObject::ot_Entity:
+        case UMLObject::ot_Datatype:
+        case UMLObject::ot_Category:
+        case UMLObject::ot_Instance:
+            return true;
+        default:
+            return false;
+    }
+}
 }  // namespace Model_Utils
 
